/* Copyright (c) 2004-2011 ActiveState Software Inc.
   See the file LICENSE.txt for licensing information. */

/* JavaScript-side control of the Code Browser. */

if (typeof(ko) == 'undefined') {
    var ko = {};
}

if (typeof(ko.codebrowser)!='undefined') {
    ko.logging.getLogger('').warn("ko.codebrowser was already loaded, re-creating it.\n");
}
ko.codebrowser = {};


(function() {

    var log = ko.logging.getLogger("codebrowser");
    //log.setLevel(ko.logging.LOG_DEBUG);

    //---- internal support routines

    // Return true iff the two file paths are equal.
    function _fequal(a, b) {
    // #if PLATFORM == "win"
        return a.toLowerCase() == b.toLowerCase();
    // #else
        return a == b;
    // #endif
    }


    //---- the controller for the code browser commands

    function _isNodeSelected() {
        return (ko.codebrowser.manager && ko.codebrowser.manager.view.selection.count != 0);
    }

    function CodeBrowserController() {
    }
    // The following two lines ensure proper inheritance (see Flanagan, p. 144).
    CodeBrowserController.prototype = new xtk.Controller();
    CodeBrowserController.prototype.constructor = CodeBrowserController;

    CodeBrowserController.prototype.destructor = function() {
    }

    CodeBrowserController.prototype.is_cmd_copy_enabled = _isNodeSelected;
    CodeBrowserController.prototype.do_cmd_copy = function() {
        ko.codebrowser.manager.copySelection();
    }


    function CodeIntelController() {
    }

    // The following two lines ensure proper inheritance (see Flanagan, p. 144).
    CodeIntelController.prototype = new xtk.Controller();
    CodeIntelController.prototype.constructor = CodeIntelController;

    CodeIntelController.prototype.destructor = function() {
    }

    CodeIntelController.prototype.is_cmd_codeBrowserGoToDefinition_enabled = _isNodeSelected;
    CodeIntelController.prototype.do_cmd_codeBrowserGoToDefinition = function() {
        ko.codebrowser.manager._jumpToSymbol();
    }


    CodeIntelController.prototype.is_cmd_codeBrowserLocate_enabled = function() {
        if (!ko.codeintel.isActive || !ko.views.manager) return false;
        var view = ko.views.manager.currentView;
        return (view && view.scimoz && view.koDoc &&
                view.isCICitadelStuffEnabled);
    }

    CodeIntelController.prototype.do_cmd_codeBrowserLocate = function() {
        ko.codebrowser.locateCurrentScope();
    }


    //---- the UI Manager for the Code Browser (koICodeBrowserManager)

    this.CBManager = function CodeBrowserManager(treeId)
    {
        log.debug("CBManager(treeId='"+treeId+"')");
        try {
            this.widgets = new Object();
            this.widgets.sortByButton = document.getElementById("codebrowser-sort-by-button");
            this.widgets.filterTextbox = document.getElementById("codebrowser-filter-textbox");
            this.widgets.tree = document.getElementById("codebrowser-tree");

            // Attach the nsITreeView implementation to the <tree/>.
            this.view = Components.classes["@activestate.com/koCodeBrowserTreeView;1"]
                                  .createInstance(Components.interfaces.koICodeBrowserTreeView);
            this.view.setCodeBrowserMgr(this);
            this.boxObject = this.widgets.tree.treeBoxObject;
            this.boxObject.view = this.view;

            // Restore preferences.
            this.view.restorePrefs();

            // Restore sort order.
            var key = null;
            if (ko.prefs.hasStringPref("codeintel_sort_codebrowser_by")) {
                key = ko.prefs.getStringPref("codeintel_sort_codebrowser_by");
            }
            if (! this.sortBy(key)) {
                var fallback = "file-order";
                log.warn("invalid Code Browser sort order key from "+
                                     "prefs ('"+key+"'): falling back to '"+
                                     fallback+"'");
                this.sortBy(fallback);
            }

            // Setup controllers.
            this.cbController = new CodeBrowserController();
            this.widgets.tree.controllers.insertControllerAt(0, this.cbController);
            this.ciController = new CodeIntelController();
            window.controllers.insertControllerAt(0, this.ciController);
            parent.controllers.insertControllerAt(0, this.ciController);

            // Setup event listeners.
            var this_ = this;
            this._on_current_view_language_changed = function(event) {
                var view = event.originalTarget;
                this_.view.change_buf_lang(view.koDoc.ciBuf);
                window.setTimeout(function() {
                    window.updateCommands('codebrowser');
                    parent.updateCommands('codebrowser');
                }, 0);
            }
            this._on_view_document_attached = function(event) {
                var view = event.originalTarget;
                if (view.getAttribute("type") != "editor") {
                    return;
                }
                this_.view.add_buf(view.koDoc.ciBuf);
                window.setTimeout(function() {
                    window.updateCommands('codebrowser');
                    parent.updateCommands('codebrowser');
                }, 0);
            }
            this._on_view_document_detaching = function(event) {
                var view = event.originalTarget;
                if (view.getAttribute("type") != "editor") {
                    return;
                }
                this_.view.remove_buf(view.koDoc.ciBuf);
                window.setTimeout(function() {
                    window.updateCommands('codebrowser');
                    parent.updateCommands('codebrowser');
                }, 0);
            }
            parent.addEventListener('current_view_language_changed',
                this._on_current_view_language_changed, false);
            parent.addEventListener('view_document_attached', this._on_view_document_attached, false);
            parent.addEventListener('view_document_detaching', this._on_view_document_detaching, false);

            this._mouseMoveTimer = null;

            // When the code browser starts up there may already be open files
            // in the current Komodo window.
            this._showTab();
            window.setTimeout(function (this_) {
                this_._addCurrentDocuments();
            }, 2000, this);
        } catch(ex) {
            Components.utils.reportError(ex);
            log.exception(ex);
        }
    }
    this.CBManager.prototype.constructor = this.CBManager;

    this.CBManager.prototype.QueryInterface = function(iid) {
        if (!iid.equals(Components.interfaces.koICodeBrowserManager) &&
            !iid.equals(Components.interfaces.nsISupports)) {
            throw Components.results.NS_ERROR_NO_INTERFACE;
        }
        return this;
    }

    this.CBManager.prototype.finalize = function() {
        log.debug("CBManager.finalize()");
        try {
            this.view.savePrefs();
            this.view.finalize();

            ko.prefs.setStringPref("codeintel_sort_codebrowser_by",
                                 this.widgets.sortByButton.getAttribute("sortBy"));
            // No point in hiding the tab at this stage - we're shutting down.
            //this._hideTab();

            parent.removeEventListener("view_document_attached",
                this._on_view_document_attached, false);
            parent.removeEventListener("view_document_detaching",
                this._on_view_document_detaching, false);
            parent.removeEventListener("current_view_language_changed",
                this._on_current_view_language_changed, false);

            this.widgets.tree.controllers.removeController(this.cbController);
            window.controllers.removeController(this.ciController);
            parent.controllers.removeController(this.ciController);

            // destroy the tree view
            this.boxObject.view = null;
            this.view.setCodeBrowserMgr(null);
            this.view = null;
        } catch(ex) {
            log.exception(ex);
        }
    }

    this.CBManager.prototype._addCurrentDocuments = function() {
        log.debug("CBManager._addCurrentDocuments()");

        var i, doc, buf = 42;
        var editorViews = ko.views.manager.topView.getViewsByType(true, "editor");
        for (i = 0; i < editorViews.length; i++) {
            doc = editorViews[i].koDoc;
            buf = doc.ciBuf;
            if (buf && ko.codeintel.is_citadel_lang(buf.lang)) {
                this.view.add_buf(buf);
                ko.codeintel.scan_document(doc, 0, true);
            }

        }
    }

    // Show the Code Browser tab.
    this.CBManager.prototype._showTab = function() {
        log.debug("CBManager._showTab()");

        var tab = window.frameElement.tab;
        if (tab.hasAttribute("collapsed"))
            tab.removeAttribute("collapsed");
        // <tabbox> Ctrl+Tab handling uses 'hidden' attribute.
        if (tab.hasAttribute("hidden"))
            tab.removeAttribute("hidden");
    }

    // Hide the Code Browser tab.
    this.CBManager.prototype._hideTab = function() {
        log.debug("CBManager._hideTab()");

        var tabbox = window.frameElement.tabbox;
        var tab = window.frameElement.tab;
        if (tabbox.selectedTab == tab) {
            var alternate = tab.previousSibling;
            if (!alternate)
                alternate = tab.nextSibling;
            if (alternate)
                tabbox.selectedTab = alternate;
        }
        tab.setAttribute("collapsed", "true");
        // <tabbox> Ctrl+Tab handling uses 'hidden' attribute.
        tab.setAttribute("hidden", "true");
    }

    this.CBManager.prototype.onSelect = function(event) {
        try {
            ko.codebrowser.manager.updateCommands();
        } catch(ex) {
            log.exception(ex);
        }
    }

    this.CBManager.prototype.onFilterKeypress = function(event) {
        try {
            if (event.keyCode == event.DOM_VK_TAB && !event.ctrlKey) {
                event.cancelBubble = true;
                event.stopPropagation();
                event.preventDefault();
                ko.codebrowser.manager.widgets.tree.focus();
                return;
            } else if (event.keyCode == event.DOM_VK_ESCAPE) {
                if (this.widgets.filterTextbox.value != '') {
                    this.widgets.filterTextbox.value = '';
                    ko.codebrowser.manager.updateFilter();
                    event.cancelBubble = true;
                    event.stopPropagation();
                    event.preventDefault();
                }
                return;
            }
        } catch (e) {
            log.exception(e);
        }
    }

    this.CBManager.prototype.updateCommands = function() {
        // update commands when the selection changes.
        ko.commands.updateCommand('cmd_copy', null, this.cbController);
        ko.commands.updateCommand('cmd_codeBrowserGoToDefinition', null, this.ciController);
    }

    this.CBManager.prototype.focus = function() {
        log.debug("CBManager.focus()");
        try {
            // Pass the focus onto the <tree/>.
            // It is important to do this because this focus handling is used to
            // get ko.uilayout.toggleTab() to work properly.
            this.widgets.tree.focus();
        } catch(ex) {
            log.exception(ex);
        }
    }

    this.CBManager.prototype._jumpToSymbol = function() {
        log.debug("CBManager._jumpToSymbol()");
        try {
            var path = this.view.currentNodeFilePath;
            if (path == null) {
                return;
            }
            ko.history.note_curr_loc();
            var line = this.view.currentNodeLine;

            if (path.slice(0, 9) == "<Unsaved>") {
                //XXX Komodo's view system does not currently provide a simple
                //    method to open or get a handle on the view for a given
                //    untitled document. We therefore have to hack it ourselves.
                path = path.slice(10);
                var views = ko.views.manager.topView.getViews(true);
                var v;
                for (var i = 0; i < views.length; i++) {
                    v = views[i];
                    if (v.getAttribute("type") == "editor"
                        && v.koDoc.isUntitled
                        && _fequal(v.koDoc.baseName, path))
                    {
                        v.makeCurrent();
                        if (line) v.currentLine = line;
                        break;
                    }
                }
            } else {
                var uri = ko.uriparse.pathToURI(path);
                if (line) {
                    ko.views.manager.doFileOpenAtLineAsync(uri, line, 'editor',
                                                           null, -1,
                                                           function(view) {
                        // Bug 32471: scroll line to a consistent position: 30% down
                        // from top. (This is a SciMoz recipe to scroll a given line
                        // to 30% down from the top of the viewport, properly
                        // accounting for folding and line wrapping.)

                        // XXX : Move this into the doFileOpenAtLine code.
                        var scimoz = view.scimoz;
                        scimoz.ensureVisible(line);
                        var want2BNLinesFromTop = Math.round(scimoz.linesOnScreen*0.30);
                        var wantTopLine2B = Math.max(0,
                            scimoz.visibleFromDocLine(line) - want2BNLinesFromTop);
                        var scrollBy = wantTopLine2B - scimoz.firstVisibleLine;
                        //dump("- Want line "+line+" (considering folding: "
                        //     +scimoz.visibleFromDocLine(line)+") to be "
                        //     +want2BNLinesFromTop+" lines (30%) from top.\n"
                        //     +"  I.e. want top line to be "+
                        //     scimoz.docLineFromVisible(wantTopLine2B)
                        //     +" (considering folding: "+wantTopLine2B+"),\n"
                        //     +"  therefore need to scroll "+scrollBy+" lines.\n");
                        scimoz.lineScroll(0, scrollBy);
                    });
                } else {
                    ko.views.manager.doFileOpenAsync(uri);
                }
            }
        } catch(ex) {
            log.exception(ex);
        }
    }


    this.CBManager.prototype.onTreeKeyPress = function(event) {
        log.debug("CBManager.onTreeKeyPress(event)");
        try {
            if (event.keyCode == event.DOM_VK_TAB && !event.ctrlKey)
            {
                event.cancelBubble = true;
                event.stopPropagation();
                event.preventDefault();
                window.setTimeout("ko.codebrowser.manager.focusFilterBox();", 0);
                return;
            }
            if (event.keyCode == event.DOM_VK_ENTER
                || event.keyCode == event.DOM_VK_RETURN)
            {
                event.cancelBubble = true;
                event.stopPropagation();
                event.preventDefault();
                ko.codebrowser.manager._jumpToSymbol();
            }
        } catch(ex) {
            log.exception(ex);
        }
    }

    this.CBManager.prototype.focusFilterBox = function() {
        this.widgets.filterTextbox.focus();
    }


    // Create instance of component given contractId and iid (as string).
    this.CBManager.prototype._createInstance = function(contractId, iidName) {
            var iid = Components.interfaces[iidName];
            return Components.classes[contractId].createInstance(iid);
    }
    // Get service given contractId and iid (as string).
    this.CBManager.prototype._getService = function(contractId, iidName) {
            var iid = Components.interfaces[iidName];
            return Components.classes[contractId].getService(iid);
    }

    this.CBManager.prototype._copyToClipboard = function(text) {
        // Copy link/image url to clipboard.
        // Get clipboard.
        var clipboard = this._getService("@mozilla.org/widget/clipboard;1",
                                         "nsIClipboard");

        // Create tranferable that will transfer the text.
        var transferable = this._createInstance("@mozilla.org/widget/transferable;1",
                                                "nsITransferable");

        if (clipboard && transferable) {
            transferable.addDataFlavor("text/unicode");
            // Create wrapper for text.
            var data = this._createInstance("@mozilla.org/supports-string;1",
                                            "nsISupportsString");
            if (data) {
                data.data = text;
                transferable.setTransferData("text/unicode", data, text.length * 2);
                // Put on clipboard.
                clipboard.setData(transferable, null,
                                  Components.interfaces.nsIClipboard.kGlobalClipboard);
            }
        }
    }

    this.CBManager.prototype.copySelection = function() {
        try {
            var currentIndex = this.view.selection.currentIndex;
            if (currentIndex == -1) {
                return;
            }
            var column = this.boxObject.columns.getNamedColumn("codebrowser-tree-node");
            var text = this.view.getCellText(currentIndex, column);
            this._copyToClipboard(text);
        } catch (e) {
            log.exception(e);
        }
    }


    this.CBManager.prototype.onDblClick = function(event) {
        log.debug("CBManager.onDblClick(event)");
        try {
            this._jumpToSymbol();
        } catch(ex) {
            log.exception(ex);
        }
    }

    // Let the Code Browser tree view know if the mouse is over a specific
    // node.  The view can use this info to show more information
    // for that node.
    this.CBManager.prototype._handleMouseDwell = function(x, y) {
        try {
            var rowObj = new Object();
            var colObj = new Object();
            var childElementObj = new Object();
            this.boxObject.getCellAt(x, y, rowObj, colObj, childElementObj);
            var row = rowObj.value;
            var col = colObj.value;
            var childElement = childElementObj.value;

            // assert col == "codebrowser-tree-node"
            ko.codebrowser.manager.view.mouseOverNode(row, childElement);

            this._mouseMoveTimer.free();
            this._mouseMoveTimer = null;
        } catch(ex) {
            log.exception(ex);
        }
    }
    this.CBManager.prototype.onMouseMove = function(event) {
        log.debug("CBManager.onMouseMove(event)");
        try {
            if (event.originalTarget.localName == "treechildren") {
                if (this._mouseMoveTimer) {
                    this._mouseMoveTimer.stopTimeout();
                    this._mouseMoveTimer.free();
                    this._mouseMoveTimer = null;
                }
                this._mouseMoveTimer = new ko.objectTimer(this, this._handleMouseDwell,
                                                       [event.clientX, event.clientY]);
                this._mouseMoveTimer.startTimeout(100);
            }
        } catch(ex) {
            log.exception(ex);
        }
    }
    this.CBManager.prototype.onMouseOut = function(event) {
        log.debug("CBManager.onMouseOut(event)");
        try {
            window.setTimeout("ko.codebrowser.manager.view.mouseOverNode(-1, '');", 50);
        } catch(ex) {
            log.exception(ex);
        }
    }

    // Set a new sort order for the code browser.
    // Returns true iff successful.
    this.CBManager.prototype.sortBy = function(key) {
        log.debug("CBManager.sortBy(key='"+key+"')");
        try {
            this._validSortByKeys = ["file-order",
                                     "name"];
            var valid = false;
            for (var i = 0; i < this._validSortByKeys.length; ++i) {
                if (key == this._validSortByKeys[i]) {
                    valid = true;
                    break;
                }
            }

            if (valid) {
                this.view.sortBy(key);
                this.widgets.sortByButton.setAttribute("sortBy", key);
            }
            return valid;
        } catch(ex) {
            log.exception(ex);
            return false;
        }
    }


    this.CBManager.prototype.haveFilesInWS = function(value) {
        log.debug("CBManager.haveFilesInWS(value='"+value+"')");
        try {
            if (value) {
                if (this.widgets.sortByButton.hasAttribute("disabled"))
                    this.widgets.sortByButton.removeAttribute("disabled");
                if (this.widgets.filterTextbox.hasAttribute("disabled"))
                    this.widgets.filterTextbox.removeAttribute("disabled");
                // Ensure that there is a selection in the tree.
                if (!this.view.selection.count) { // there is no selection
                    this.view.selection.select(0);
                }
            } else {
                this.widgets.sortByButton.setAttribute("disabled", "true");
                this.widgets.filterTextbox.setAttribute("disabled", "true");
            }
        } catch(ex) {
            log.exception(ex);
        }
    }


    this.CBManager.prototype.updateFilter = function() {
        log.debug("CBManager.updateFilter()");
        try {
            var textbox = this.widgets.filterTextbox;
            var filterPattern = textbox.value;
            try {
                this.view.setFilter(filterPattern);
                if (textbox.hasAttribute("error")) {
                    textbox.removeAttribute("error");
                    textbox.setAttribute("tooltiptext",
                        textbox.getAttribute("basetooltiptext"));
                }
            } catch (ex) { // Means an invalid regex pattern.
                textbox.setAttribute("error", "true");
                textbox.setAttribute("tooltiptext",
                    textbox.getAttribute("basetooltiptext") + " -- error in pattern: " + ex);
            }
            if (this.view.rowCount) {
                this.view.selection.select(0);
            }
            this.updateCommands();
        } catch(ex) {
            log.exception(ex);
        }
    }

    //---- public routines
    
    this.initialize = function CodeBrowser_Initialize()
    {
        log.debug("initialize()");
        try {
            ko.codebrowser.manager = new ko.codebrowser.CBManager("codebrowser-tree");
            window.updateCommands('codebrowser');
            ko.main.addWillCloseHandler(ko.codebrowser.finalize);
        } catch(ex) {
            log.exception(ex);
        }
    }
    
    this.finalize = function CodeBrowser_Finalize()
    {
        log.debug("finalize()");
        try {
            if (ko.codebrowser.manager) {
                ko.codebrowser.manager.finalize();
                ko.codebrowser.manager = null;
            }
        } catch(ex) {
            log.exception(ex);
        }
    }

    // Locate the node in the Code Browser corresponding to the current scope.
    this.locateCurrentScope = function CodeIntel_LocateCurrentScope()
    {
        log.debug("locateCurrentScope()");
        try {
            if (!ko.codeintel.isActive) return;
            var view = ko.views.manager.currentView;
            if (view && view.scimoz && view.koDoc) {
                var scimoz = view.scimoz;
                var located = ko.codebrowser.manager.view.locateScope(view.koDoc.ciBuf, scimoz.currentPos);
                if (located) {
                    var filter = document.getElementById("codebrowser-filter-textbox");
                    filter.value = "";
                    ko.uilayout.ensureTabShown(window.frameElement, true);
                }
                if (!located) {
                    var line = scimoz.lineFromPosition(scimoz.currentPos)+1;
                    ko.dialogs.alert("Could not find a node corresponding to the "
                                 + "scope at " + view.koDoc.baseName + ":"
                                 + line + " in the Code Browser.");
                }
            }
        } catch(ex) {
            log.exception(ex);
        }
    }

}).apply(ko.codebrowser);

window.addEventListener("focus", function(event) {
    if (event.originalTarget == window) {
        ko.codebrowser.manager.focus();
    }
}, false);

/**
 * @deprecated since 7.0
 */
ko.logging.globalDeprecatedByAlternative("gCodeBrowserMgr", "ko.codebrowser.manager");
ko.logging.globalDeprecatedByAlternative("gCodeBrowserCtrl", "ko.codebrowser.manager.cbController");
ko.logging.globalDeprecatedByAlternative("CodeIntel_LocateCurrentScope", "ko.codebrowser.locateCurrentScope");
