const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;
const Cu = Components.utils;

Cu.import("resource://gre/modules/PluralForm.jsm");

Cu.import("resource://services/sso.js");

// page consts

const INTRO_PAGE                    = 0;
const SSO_SETUP_PAGE                = 1;
const NEW_ACCOUNT_PP_PAGE           = 2;
const EXISTING_ACCOUNT_PP_PAGE      = 3;
const OPTIONS_PAGE                  = 4;
const OPTIONS_CONFIRM_PAGE          = 5;
const SETUP_SUCCESS_PAGE            = 6;

var gWeaveSetup = {
  _settingUpNew: false,
  
  get _usingMainServers() {
    if (this._settingUpNew)
      return document.getElementById("serverType").selectedItem.value == "main";

    return document.getElementById("existingServerType").selectedItem.value == "main";
  },

  status: { password: false, email: false, server: false},

  get bundle() {
    delete this.bundle;
    return this.bundle = document.getElementById("weavePrefStrings");
  },

  get wizard() {
    delete this.wizard;
    return this.wizard = document.getElementById("accountSetup");
  },

  init: function () {
    let obs = [
      ["weave:service:changepph:finish", "onResetPP"],
      ["weave:service:verify-login:start",  "onLoginStart"],
      ["weave:service:verify-login:error",  "onLoginEnd"],
      ["weave:service:verify-login:finish", "onLoginEnd"],
      ["services:sso:login-successful", "onSsoLogin"],
      ["services:sso:login-failed", "onSsoLoginFailed"]];

    // Add the observers now and remove them on unload
    let weavePrefs = this;
    let addRem = function(add) obs.forEach(function([topic, func])
      Weave.Svc.Obs[add ? "add" : "remove"](topic, weavePrefs[func], weavePrefs));
    addRem(true);
    window.addEventListener("unload", function() addRem(false), false);
    
    if (window.arguments && window.arguments[0] == true) {
      // we're resetting sync
      this._resettingSync = true;
      this.wizard.pageIndex = OPTIONS_PAGE;
    }
    else {
      Weave.Svc.Prefs.set("firstSync", "notReady");
    }

    this.wizard.getButton("extra1").label = this.bundle.getString("button.syncOptions.label");

    // Remember these values because the options pages change them temporarily.
    this._nextButtonLabel = this.wizard.getButton("next").label;
    this._nextButtonAccesskey = this.wizard.getButton("next")
                                           .getAttribute("accesskey");
    this._backButtonLabel = this.wizard.getButton("back").label;
    this._backButtonAccesskey = this.wizard.getButton("back")
                                           .getAttribute("accesskey");
    // Wanted to put this in onPageShow, but onload fires later, and so the
    // event observers didn't trigger. :(
    this.trySsoLogin();
  },
  
  trySsoLogin: function (email, password) {
    if ((email && password) || SSOSvc.checkSetup()) {
      if (!SSOSvc.loggedIn) {
        SSOSvc.login(email, password);
        this.onSsoLoggingIn();
      } else {
        this.onSsoLogin();
      }
    } else {
      this.onSsoLoginFailed();
    }
  },
  
  onSsoLoggingIn: function () {
    let el, msg = this.bundle.getString("ssoSetupLoggingIn");
    switch(this.wizard.pageIndex) {
      case INTRO_PAGE:
        el = document.getElementById("introLoginStatusRow")
        break;
      case SSO_SETUP_PAGE:
        el = document.getElementById("ssoSetupStatusRow");
        break;
    }
    this._setFeedback(el, 'active', msg);
  },
  
  onSsoLogin: function () {
    if (!SSOSvc.loggedIn || !SSOSvc.userId || !SSOSvc.sessionKey)
      throw "SSO not logged in!"
    
    if (!Weave.Service.username || !Weave.Service.password) {
      // SsoConnector should have taken of there it is, our callback was
      // probably faster.
      Weave.Service.username = SSOSvc.userId;
      Weave.Service.password = SSOSvc.sessionKey;
    }
    
    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        this._setFeedback(document.getElementById("introLoginStatusRow"), true);
        this.wizard.canAdvance = true;
        break;
      case SSO_SETUP_PAGE:
        this._setFeedback(document.getElementById("ssoSetupStatusRow"), true);
        let this_ = this;
        // Allow other login-successful callbacks to be executed before we try
        // to contact the Sync Server.
        window.setTimeout(function() this_.goToPassphrasePage(), 0);
        break;
    }
  },
  
  onSsoLoginFailed: function () {
    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        this._setFeedback(document.getElementById("introLoginStatusRow"), true);
        this.wizard.canAdvance = true;
        break;
      case SSO_SETUP_PAGE:
        this._setFeedback(document.getElementById("ssoSetupStatusRow"), false,
                          this.bundle.getString("ssoSetupLoginFailed"));
        break;
    }
  },
  
  onResetPP: function () {
    document.getElementById("existingPassphrase").value = Weave.Utils.hyphenatePassphrase(Weave.Service.passphrase);
    this.wizard.advance();
  },

  onLoginStart: function () {
    this.toggleLoginFeedback(false);
  },

  onLoginEnd: function () {
    this.toggleLoginFeedback(true);
  },

  toggleLoginFeedback: function (stop) {
    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        if (stop) {
          this._setFeedback(document.getElementById("introLoginStatusRow"), true);
        } else {
          this._setFeedback(document.getElementById("introLoginStatusRow"), 'active', ' ');
        }
        break;
      case EXISTING_ACCOUNT_PP_PAGE:
        document.getElementById("passphrase-throbber").hidden = stop;
        feedback = document.getElementById("existingPassphraseFeedbackBox");
        if (stop) {
          let success = Weave.Status.login == Weave.LOGIN_SUCCEEDED;
          this._setFeedbackMessage(feedback, success, Weave.Status.login);
          document.getElementById("passphraseHelpBox").hidden = success;
        }
        else
          this._setFeedbackMessage(feedback, true);

        break;
    }
  },

  setupInitialSync: function () {
    let action = document.getElementById("mergeChoiceRadio").selectedItem.id;
    switch (action) {
      case "resetClient":
        // if we're not resetting sync, we don't need to explicitly
        // call resetClient
        if (!this._resettingSync)
          return;
        // otherwise, fall through
      case "wipeClient":
      case "wipeRemote":
        Weave.Svc.Prefs.set("firstSync", action);
        break;
    }
  },

  onPassphraseKeyUp: function (event) {
    this.checkFields();
  },

  // fun with validation!
  checkFields: function () {
    this.wizard.canAdvance = this.readyToAdvance();
  },

  readyToAdvance: function () {
    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        return true;
      case SSO_SETUP_PAGE:
        return true;
      case EXISTING_ACCOUNT_PP_PAGE:
        return document.getElementById("existingPassphrase").value != "";
    }
    // we probably shouldn't get here
    return true;
  },

  checkAccount: function() {
    delete this._checkAccountTimer;
    let value = Weave.Utils.normalizeAccount(document.getElementById("weaveEmail").value);
    if (!value) {
      this.status.email = false;
      this.checkFields();
      return;
    }

    let re = /^(([^<>()[\]\\.,;:\s@\"]+(\.[^<>()[\]\\.,;:\s@\"]+)*)|(\".+\"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    let feedback = document.getElementById("emailFeedbackRow");
    let valid = re.test(value);

    let str = "";
    if (!valid) {
      str = "invalidEmail.label";
    } else {
      let availCheck = Weave.Service.checkAccount(value);
      valid = availCheck == "available";
      if (!valid) {
        if (availCheck == "notAvailable")
          str = "usernameNotAvailable.label";
        else
          str = availCheck;
      }
    }

    this._setFeedbackMessage(feedback, valid, str);
    this.status.email = valid;
    if (valid)
      Weave.Service.account = value;
    this.checkFields();
  },

  onPassphraseGenerate: function () {
    let passphrase = Weave.Utils.generatePassphrase();
    Weave.Service.passphrase = passphrase;
    let el = document.getElementById("weavePassphrase");
    el.value = Weave.Utils.hyphenatePassphrase(passphrase);
  },
  
  _fixHeaderLabel: function () {
    // Make the header label text a text node rather than an attribute on the
    // <label>. That way it will wrap rather than stretch its container. We're
    // monkey-patching this in here to avoid creating a custom XBL binding for
    // such a minor change in the wizard's anonymous XUL.
    let header_label = document.getAnonymousElementByAttribute(
      this.wizard._wizardHeader, "class", "wizard-header-label");
    let text_node = header_label.childNodes[0];
    if (!text_node) {
      text_node = document.createTextNode("");
      header_label.appendChild(text_node);
    }
    text_node.nodeValue = header_label.value;
    header_label.removeAttribute("value");
  },

  onPageShow: function() {
    this._fixHeaderLabel();

    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        //this.wizard.getButton("next").hidden = true;
        this.wizard.getButton("back").hidden = true;
        this.wizard.getButton("extra1").hidden = true;
        // Will be set to true, once we catch a sso login event.
        this.wizard.readyToAdvance = false;
        
        break;
      case NEW_ACCOUNT_PP_PAGE:
        document.getElementById("saveSyncKeyButton").focus();
        let el = document.getElementById("weavePassphrase");
        if (!el.value)
          this.onPassphraseGenerate();
        this.checkFields();
        break;
      case SSO_SETUP_PAGE:
        this.wizard.getButton("extra1").hidden = true;
        break;
      case EXISTING_ACCOUNT_PP_PAGE:
        this.wizard.getButton("extra1").hidden = false;
        this.checkFields();
        break;
      case SETUP_SUCCESS_PAGE:
        this.wizard.canRewind = false;
        this.wizard.getButton("back").hidden = true;
        this.wizard.getButton("next").hidden = true;
        this.wizard.getButton("cancel").hidden = true;
        this.wizard.getButton("finish").hidden = false;
        this._handleSuccess();
        break;
      case OPTIONS_PAGE:
        this.wizard.canRewind = false;
        this.wizard.canAdvance = true;
        if (!this._resettingSync) {
          this.wizard.getButton("next").label =
            this.bundle.getString("button.syncOptionsDone.label");
          this.wizard.getButton("next").removeAttribute("accesskey");
        }
        this.wizard.getButton("next").hidden = false;
        this.wizard.getButton("back").hidden = true;
        this.wizard.getButton("cancel").hidden = !this._resettingSync;
        this.wizard.getButton("extra1").hidden = true;
        document.getElementById("syncComputerName").value = Weave.Clients.localName;
        document.getElementById("syncOptions").collapsed = this._resettingSync;
        document.getElementById("mergeOptions").collapsed = this._settingUpNew;
        break;
      case OPTIONS_CONFIRM_PAGE:
        this.wizard.canRewind = true;
        this.wizard.canAdvance = true;
        this.wizard.getButton("back").label =
          this.bundle.getString("button.syncOptionsCancel.label");
        this.wizard.getButton("back").removeAttribute("accesskey");
        this.wizard.getButton("back").hidden = this._resettingSync;
        this.wizard.getButton("next").hidden = false;
        this.wizard.getButton("finish").hidden = true;
        break;
    }
  },
  
  goToPassphrasePage: function() {
    if (!SSOSvc.loggedIn || !SSOSvc.userId || !SSOSvc.sessionKey)
      throw "sso must be logged in!"
    
    this.wizard.canAdvance = false;
    // Check if Sync accepts our login data.
    try {
      if (Weave.Service.verifyLogin()) {
        // We already have a valid passphrase set. Unlikely, but we're finished.
        this.wizard.pageIndex = SETUP_SUCCESS_PAGE;
      } else if (Weave.Status.login != Weave.LOGIN_FAILED_NO_PASSPHRASE
        && Weave.Status.login != Weave.LOGIN_FAILED_INVALID_PASSPHRASE) {
        // Invalid username/password or network error.
      } else {
        // Login failed because of missing or new passphrase-
        // Try a random passphrase. If it succeeds, the account is being set
        // up for the first time and the random passphrase is now their
        // passphrase. If it fails, there already is a passphrase in the
        // account and we have to prompt the user for it.
        Weave.Service.passphrase = Weave.Utils.generatePassphrase();
        if (Weave.Service.verifyLogin()) {
          this._settingUpNew = true;
          // the random passphrase has been set on the server and client
          let el = document.getElementById("weavePassphrase");
          el.value = Weave.Utils.hyphenatePassphrase(Weave.Service.passphrase);
          this.wizard.pageIndex = NEW_ACCOUNT_PP_PAGE;
        } else {
          // Random passphrase was wrong- delete it and prompt user for pp
          Weave.Service.passphrase = "";
          this.wizard.pageIndex = EXISTING_ACCOUNT_PP_PAGE;
        }
      }
    } catch(e) {
      // Probably a network error, see #591102. Make sure to check for
      // Weave.Status.login in calling function.
      Weave.Service._log.warn("Uncaught exception in verifyLogin: " + e);
      this.displaySyncServerError();
    }
    return false;
  },
  
  displaySyncServerError: function() {
    let el;
    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        el = document.getElementById("introLoginStatusRow");
        break;
      case SSO_SETUP_PAGE:
        el = document.getElementById("ssoSetupStatusRow");
        break;
    }
    if (el) {
      this._setFeedback(el, false, this.bundle.getString("syncServerError"));
    }
  },

  onWizardAdvance: function () {
    switch (this.wizard.pageIndex) {
      case INTRO_PAGE:
        if (SSOSvc.loggedIn)
          this.goToPassphrasePage();
        else
          this.wizard.pageIndex = SSO_SETUP_PAGE;
        return false;
      case SSO_SETUP_PAGE:
        let email = document.getElementById("setupSsoEmailField").value;
        let password = document.getElementById("setupSsoPasswordField").value;
        this.trySsoLogin(email, password);
        return false;
      case NEW_ACCOUNT_PP_PAGE:
        this.wizard.pageIndex = SETUP_SUCCESS_PAGE;
        return false;
        break;
      case EXISTING_ACCOUNT_PP_PAGE:
        let pp = document.getElementById("existingPassphrase").value;
        Weave.Service.passphrase = Weave.Utils.normalizePassphrase(pp);
        if (Weave.Service.login())
          this.wizard.pageIndex = SETUP_SUCCESS_PAGE;
        return false;
      case OPTIONS_PAGE:
        let desc = document.getElementById("mergeChoiceRadio").selectedIndex;
        // No confirmation needed on new account setup or merge option
        // with existing account.
        if (this._settingUpNew || (!this._resettingSync && desc == 0))
          return this.returnFromOptions();
        return this._handleChoice();
      case OPTIONS_CONFIRM_PAGE:
        if (this._resettingSync) {
          this.onWizardFinish();
          window.close();
          return false;
        }
        return this.returnFromOptions();
    }
    return true;
  },

  onWizardBack: function () {
    switch (this.wizard.pageIndex) {
      case SSO_SETUP_PAGE:
        this.wizard.pageIndex = INTRO_PAGE;
        return false;
      case OPTIONS_CONFIRM_PAGE:
        // Backing up from the confirmation page = resetting first sync to merge.
        document.getElementById("mergeChoiceRadio").selectedIndex = 0;
        return this.returnFromOptions();
    }
    return true;
  },

  onWizardFinish: function () {
    this.setupInitialSync();

    if (!this._resettingSync) {
      function isChecked(element) {
        return document.getElementById(element).hasAttribute("checked");
      }

      let prefs = ["engine.apicatalogs", "engine.keybindings",
        "engine.kopasswords", "engine.koprefs", "engine.projecttemplates",
        "engine.schemes", "engine.templates", "engine.tools"];
      for (let i = 0;i < prefs.length;i++) {
        Weave.Svc.Prefs.set(prefs[i], isChecked(prefs[i]));
      }
      this._handleNoScript(false);
      if (Weave.Svc.Prefs.get("firstSync", "") == "notReady")
        Weave.Svc.Prefs.reset("firstSync");

      Weave.Service.persistLogin();
      Weave.Svc.Obs.notify("weave:service:setup-complete");
      /* XXX This opens Firefox landing pages. Disable for now.
      if (this._settingUpNew)
        gWeaveCommon.openFirstClientFirstrun();
      else
        gWeaveCommon.openAddedClientFirstrun(); */
    }

    if (!Weave.Service.isLoggedIn)
      Weave.Service.login();

    Weave.Service.syncOnIdle(1);
  },

  onWizardCancel: function () {
    if (this._resettingSync)
      return;

    if (this.wizard.pageIndex == SETUP_SUCCESS_PAGE) {
      this.onWizardFinish();
      return;
    }
    this._handleNoScript(false);
    Weave.Service.startOver();
  },

  onSyncOptions: function () {
    this._beforeOptionsPage = this.wizard.pageIndex;
    this.wizard.pageIndex = OPTIONS_PAGE;
  },

  returnFromOptions: function() {
    this.wizard.getButton("next").label = this._nextButtonLabel;
    this.wizard.getButton("next").setAttribute("accesskey",
                                               this._nextButtonAccesskey);
    this.wizard.getButton("back").label = this._backButtonLabel;
    this.wizard.getButton("back").setAttribute("accesskey",
                                               this._backButtonAccesskey);
    this.wizard.getButton("cancel").hidden = false;
    this.wizard.getButton("extra1").hidden = false;
    this.wizard.pageIndex = this._beforeOptionsPage;
    return false;
  },

  _disabledSites: [],
  get _remoteSites() {
    return [Weave.Service.serverURL, "https://api-secure.recaptcha.net"];
  },
  
  _handleNoScript: function (addExceptions) {
    // if NoScript isn't installed, or is disabled, bail out.
    let ns = Cc["@maone.net/noscript-service;1"];
    if (ns == null)
      return;

    ns = ns.getService().wrappedJSObject;
    if (addExceptions) {
      this._remoteSites.forEach(function(site) {
        site = ns.getSite(site);
        if (!ns.isJSEnabled(site)) {
          this._disabledSites.push(site); // save status
          ns.setJSEnabled(site, true); // allow site
        }
      }, this);
    }
    else {
      this._disabledSites.forEach(function(site) {
        ns.setJSEnabled(site, false);
      });
      this._disabledSites = [];
    }
  },

  _handleSuccess: function() {
    let self = this;
    function fill(id, string)
      document.getElementById(id).firstChild.nodeValue =
        string ? self.bundle.getString(string) : "";

    fill("firstSyncAction", "");
    fill("firstSyncActionWarning", "");
    if (this._settingUpNew) {
      fill("firstSyncAction", "newAccount.action.label");
      fill("firstSyncActionChange", "newAccount.change.label");
      return;
    }
    fill("firstSyncActionChange", "existingAccount.change.label");
    let action = document.getElementById("mergeChoiceRadio").selectedItem.id;
    let id = action == "resetClient" ? "firstSyncAction" : "firstSyncActionWarning";
    fill(id, action + ".change.label");
  },

  _handleChoice: function () {
    let desc = document.getElementById("mergeChoiceRadio").selectedIndex;
    document.getElementById("chosenActionDeck").selectedIndex = desc;
    switch (desc) {
      case 1:
        if (this._case1Setup)
          break;

        /* XXX Firefox specific stuff that we don't care about
        // history
        let db = Weave.Svc.History.DBConnection;

        let daysOfHistory = 0;
        let stm = db.createStatement(
          "SELECT ROUND(( " +
            "strftime('%s','now','localtime','utc') - " +
            "( " +
              "SELECT visit_date FROM moz_historyvisits " +
              "UNION ALL " +
              "SELECT visit_date FROM moz_historyvisits_temp " +
              "ORDER BY visit_date ASC LIMIT 1 " +
              ")/1000000 " +
            ")/86400) AS daysOfHistory ");

        if (stm.step())
          daysOfHistory = stm.getInt32(0);
        document.getElementById("historyCount").value =
          PluralForm.get(daysOfHistory,
                         this.bundle.getString("historyDaysCount.label"))
                    .replace("#1", daysOfHistory);

        // bookmarks
        let bookmarks = 0;
        stm = db.createStatement(
          "SELECT count(*) AS bookmarks " +
          "FROM moz_bookmarks b " +
          "LEFT JOIN moz_bookmarks t ON " +
          "b.parent = t.id WHERE b.type = 1 AND t.parent <> :tag");
        stm.params.tag = Weave.Svc.Bookmark.tagsFolder;
        if (stm.executeStep())
          bookmarks = stm.row.bookmarks;
        document.getElementById("bookmarkCount").value =
          PluralForm.get(bookmarks,
                         this.bundle.getString("bookmarksCount.label"))
                    .replace("#1", bookmarks);

        // passwords
        let logins = Weave.Svc.Login.getAllLogins({});
        document.getElementById("passwordCount").value =
          PluralForm.get(logins.length,
                         this.bundle.getString("passwordsCount.label"))
                    .replace("#1", logins.length);
        this._case1Setup = true; */
        break;
      case 2:
        if (this._case2Setup)
          break;
        let count = 0;
        function appendNode(label) {
          let box = document.getElementById("clientList");
          let node = document.createElement("label");
          node.setAttribute("value", label);
          node.setAttribute("class", "data indent");
          box.appendChild(node);
        }

        for each (let name in Weave.Clients.stats.names) {
          // Don't list the current client
          if (name == Weave.Clients.localName)
            continue;

          // Only show the first several client names
          if (++count <= 5)
            appendNode(name);
        }
        if (count > 5) {
          let label =
            PluralForm.get(count - 5,
                           this.bundle.getString("additionalClientCount.label"))
                      .replace("#1", count - 5);
          appendNode(label);
        }
        this._case2Setup = true;
        break;
    }

    return true;
  },

  // sets class and string on a feedback element
  // if no property string is passed in, we clear label/style
  _setFeedback: function (element, success, string) {
    element.hidden = !string;
    let class_;
    if (typeof success == 'boolean')
      class_ = success ? "success" : "error";
    else
      class_ = success;
    let image = element.firstChild.nextSibling.firstChild;
    image.setAttribute("status", class_);
    let label = image.nextSibling;
    label.value = string;
  },

  // shim
  _setFeedbackMessage: function (element, success, string) {
    let str = "";
    if (string) {
      try {
        str = this.bundle.getString(string);
      } catch(e) {}
  
      if (!str)
        str = Weave.Utils.getErrorString(string);
    }
    this._setFeedback(element, success, str);
  },

  QueryInterface: function (aIID) {
    if (aIID.equals(Ci.nsIWebProgressListener) ||
        aIID.equals(Ci.nsISupportsWeakReference) ||
        aIID.equals(Ci.nsISupports))
      return this;
    throw Cr.NS_NOINTERFACE;
  },

  onStateChange: function(webProgress, request, stateFlags, status) {
    // We're only looking for the end of the frame load
    if ((stateFlags & Ci.nsIWebProgressListener.STATE_STOP) == 0)
      return;
    if ((stateFlags & Ci.nsIWebProgressListener.STATE_IS_NETWORK) == 0)
      return;
    if ((stateFlags & Ci.nsIWebProgressListener.STATE_IS_WINDOW) == 0)
      return;

    // If we didn't find the captcha, assume it's not needed and move on
    if (request.QueryInterface(Ci.nsIHttpChannel).responseStatus == 404)
      this.onWizardAdvance();
  },
  onProgressChange: function() {},
  onStatusChange: function() {},
  onSecurityChange: function() {},
  onLocationChange: function () {}
}
